package org.codehaus.xfire.java;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamReader;

import org.apache.log4j.Logger;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.handler.AbstractHandler;
import org.codehaus.xfire.java.message.MessageBridge;
import org.codehaus.xfire.java.message.MessageBridgeFactory;

/**
 * Handles java services.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 18, 2004
 */
public class JavaServiceHandler
    extends AbstractHandler
{
    private static Logger logger = Logger.getLogger(JavaServiceHandler.class.getName());
    
	public static final String NAME = "java";

	/**
	 * @see org.codehaus.xfire.handler.Handler#invoke(org.codehaus.xfire.MessageContext)
	 */
	public void invoke( MessageContext context,
                        XMLStreamReader reader )
		throws XFireFault
	{
		JavaService service = (JavaService) context.getService();

		MessageBridge pipe = MessageBridgeFactory.createMessageBridge( service,
                                                                       context,
                                                                       reader,
                                                                       getXMLStreamWriter(context) );

        try
        {
            // Read in the parameters...
            List params = pipe.read();
            
            // Don't read the operation in until after reading. Otherwise
            // it won't work for document style services.
            Operation operation = pipe.getOperation();
            
            // invoke the service method...
            Object value = invokeService( operation, params, service, context );

            pipe.write( value );
        }
        catch (XFireRuntimeException e)
        {
            logger.warn("Error invoking service.", e);
            throw new XFireFault("Error invoking service.", e, XFireFault.SENDER);
        }
	}

    /**
     * @param methodName
     * @param params
     */
    private Object invokeService( Operation operation, 
                                  List params,
                                  JavaService service,
                                  MessageContext context ) throws XFireFault
    {
        Method method = operation.getMethod();
        
        try
        {
            Object serviceObject = service.getServiceObject( context );

            return method.invoke( serviceObject, params.toArray() );
        }
        catch (IllegalArgumentException e)
        {
            throw new XFireFault( "Illegal argument.", e, XFireFault.SENDER );
        }
        catch (InvocationTargetException e)
        {
            Throwable t = e.getTargetException();
            t.getMessage();
            
            if ( t instanceof XFireFault )
            {
                throw (XFireFault) t;
            }
            else if ( t instanceof Exception )
            {
                logger.warn("Error invoking service.", t);
                throw new XFireFault( (Exception) t, XFireFault.SENDER );
            }
            else
            {
                logger.warn("Error invoking service.", e);
                throw new XFireRuntimeException( "Error invoking service.", e );
            }
        }
        catch (IllegalAccessException e)
        {
            throw new XFireFault( "Couldn't access service object.", e, XFireFault.RECEIVER );
        }
    }
    
    /**
     * @see org.codehaus.xfire.handler.dom.DOMHandler#getUnderstoodHeaders()
     */
    public QName[] getUnderstoodHeaders()
    {
        return null;
    }
}
