package org.codehaus.xfire.java.mapping;

import java.util.Hashtable;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.codehaus.xfire.AbstractXFireComponent;
import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.java.JavaServiceHandler;
import org.codehaus.xfire.java.type.BooleanType;
import org.codehaus.xfire.java.type.DoubleType;
import org.codehaus.xfire.java.type.FloatType;
import org.codehaus.xfire.java.type.IntType;
import org.codehaus.xfire.java.type.LongType;
import org.codehaus.xfire.java.type.StringType;

/**
 * The default implementation of TypeMappingRegistry.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 22, 2004
 */
public class DefaultTypeMappingRegistry
    extends AbstractXFireComponent
    implements TypeMappingRegistry
{
    private static Logger logger = Logger.getLogger(JavaServiceHandler.class.getName());
    
    private Hashtable registry;
    
    private TypeMapping defaultTM;
    
    public DefaultTypeMappingRegistry()
    {
        registry = new Hashtable();
    }
    
	/**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#register(java.lang.String, org.codehaus.xfire.java.encoding.TypeMapping)
	 */
	public TypeMapping register( String encodingStyleURI, TypeMapping mapping )
	{
        TypeMapping previous = (TypeMapping) registry.get( encodingStyleURI );
       
        mapping.setEncodingStyleURI( encodingStyleURI );
        
		registry.put( encodingStyleURI, mapping );
        
        return previous;
	}

    /**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#registerDefault(org.codehaus.xfire.java.encoding.TypeMapping)
	 */
	public void registerDefault( TypeMapping mapping )
	{
		defaultTM = mapping;
	}
    
	/**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#getDefaultTypeMapping()
	 */
	public TypeMapping getDefaultTypeMapping()
	{
		return defaultTM;
	}

    /**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#getRegisteredEncodingStyleURIs()
	 */
	public String[] getRegisteredEncodingStyleURIs()
	{
		return (String[]) registry.keySet().toArray(new String[0]);
	}

    /**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#getTypeMapping(java.lang.String)
	 */
	public TypeMapping getTypeMapping( String encodingStyleURI )
	{
		return (TypeMapping) registry.get(encodingStyleURI);
	}
    
	/**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#createTypeMapping()
	 */
	public TypeMapping createTypeMapping( boolean autoTypes )
	{
		return createTypeMapping( getDefaultTypeMapping(), autoTypes );
	}

    /**
     * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#createTypeMapping(java.lang.String)
     */
    public TypeMapping createTypeMapping( String parentNamespace, boolean autoTypes )
    {
        return createTypeMapping( getTypeMapping( parentNamespace ), autoTypes );
    }
    
    protected TypeMapping createTypeMapping( TypeMapping parent, boolean autoTypes )
    {
        if ( autoTypes )
        {
        	return new AutoTypeMapping( parent );
        }
        else
        {
        	return new CustomTypeMapping( parent );
        }
    }
    
    /**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#unregisterTypeMapping(java.lang.String)
	 */
	public TypeMapping unregisterTypeMapping( String encodingStyleURI )
	{
		TypeMapping tm = (TypeMapping) registry.get(encodingStyleURI);
        registry.remove(encodingStyleURI);
        return tm;
	}
    
	/**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#removeTypeMapping(org.codehaus.xfire.java.encoding.TypeMapping)
	 */
	public boolean removeTypeMapping( TypeMapping mapping )
	{
        int n = 0;
        
        for ( Iterator itr = registry.values().iterator(); itr.hasNext(); )
        {
            if ( itr.next().equals( mapping ) )
            {
            	itr.remove();
                n++;
            }
        }
        
        return (n > 0);
	}

	/**
	 * @see org.codehaus.xfire.java.mapping.TypeMappingRegistry#clear()
	 */
	public void clear()
	{
		registry.clear();
	}

    /**
     * Load a class from the class loader.
     * 
     * @param className The name of the class.
     * @return The class.
     * @throws Exception
     */
    protected Class loadClass( String className )
        throws Exception
    {
        // Handle array'd types.
        if ( className.endsWith("[]") )
        {
            className = "[L" + className.substring(0, className.length() - 2 ) + ";";
        }
        
        return getClass().getClassLoader().loadClass( className );
    }
    
    public TypeMapping createXSDMapping()
    {
        TypeMapping tm = createTypeMapping(SOAPConstants.XSD, false);
        
        tm.register(boolean.class, new QName(SOAPConstants.XSD,"boolean"), BooleanType.class);
        tm.register(int.class, new QName(SOAPConstants.XSD,"int"), IntType.class);
        tm.register(double.class, new QName(SOAPConstants.XSD,"double"), DoubleType.class);
        tm.register(float.class, new QName(SOAPConstants.XSD,"float"), FloatType.class);
        tm.register(long.class, new QName(SOAPConstants.XSD,"long"), LongType.class);
        tm.register(String.class, new QName(SOAPConstants.XSD,"string"), StringType.class);
        tm.register(Boolean.class, new QName(SOAPConstants.XSD,"boolean"), BooleanType.class);
        tm.register(Integer.class, new QName(SOAPConstants.XSD,"int"), IntType.class);
        tm.register(Double.class, new QName(SOAPConstants.XSD,"double"), DoubleType.class);
        tm.register(Float.class, new QName(SOAPConstants.XSD,"float"), FloatType.class);
        tm.register(Long.class, new QName(SOAPConstants.XSD,"long"), LongType.class);

        register(SOAPConstants.XSD, tm);
        
        return tm;
    }
}
