package org.codehaus.xfire.java.message;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.java.JavaService;
import org.codehaus.xfire.java.Operation;
import org.codehaus.xfire.java.type.Type;
import org.codehaus.xfire.util.DepthXMLStreamReader;

/**
 * Bridges Document/Literal style messages.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class DocumentBridge
    extends AbstractMessageBridge
{
	public DocumentBridge( JavaService service, 
                           MessageContext context,
                           XMLStreamReader request,
                           XMLStreamWriter response )
    {
		super( service, context, request, response );
        
        setNamespace( getService().getDefaultNamespace() );
	}

    public List read()
    {
        List parameters = new ArrayList();
        MessageReader reader = new LiteralReader(new DepthXMLStreamReader(getRequestReader()));
        
        while( reader.hasMoreChildReaders() )
        {
            Class clazz = getOperation().getParameterClass( reader.getLocalName() );

            // TODO: support complex type mappings...
            Type type = getTypeMapping().getType( reader.getName() );

            parameters.add( type.readObject( reader.getNextChildReader() ) );
        }
        
        setOperation( findOperation( parameters.size() ) );
        return parameters;
    }


    protected Operation findOperation(int i)
    {
        for ( Iterator itr = getService().getOperations().iterator(); itr.hasNext(); )
        {
            Operation o = (Operation) itr.next();
            if ( o.getParameters().size() == i )
                return o;
        }
        
        return null;
    }

    public void write(Object value)
    {
        Type type = getTypeMapping().getType( value.getClass() );

        String ns = null;
        
        if ( type.isComplex() )
        {
            ns = type.getSchemaType().getNamespace().getURI();
        }
        else
        {
        	ns = getNamespace();
        }
        
        String name = getOperation().getOutParameterName();
        LiteralWriter writer = new LiteralWriter( getResponseWriter(), name, ns );

        type.writeObject( value, writer );
        
        writer.close();
    }
}
