package org.codehaus.xfire.java.message;

import java.util.Calendar;
import java.util.Date;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.util.DateUtils;

/**
 * LiteralWriter
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class LiteralWriter
    implements MessageWriter
{
    private XMLStreamWriter writer;
    
    private String namespace;
    
    private String name;

    private static Calendar calendar = Calendar.getInstance();

    public LiteralWriter( XMLStreamWriter writer, String name, String namespace )
    {
        this.writer = writer;
        this.namespace = namespace;
        this.name = name;
        
        try
        {
            writeStartElement();
        }
        catch ( XMLStreamException e )
        {
            // TODO: I'm still not sure what should happen here.
            throw new XFireRuntimeException("Error writing document.", e);
        }
        
    }

    private void writeStartElement() 
        throws XMLStreamException
    {
        String prefix = writer.getPrefix(namespace);
        boolean writeNS = false;
        if ( prefix == null )
        {
            prefix = "";
            writer.setPrefix(prefix, namespace);
            writeNS = true;
        }
        
        writer.writeStartElement("", name, namespace);
        
        if ( writeNS )
            writer.writeDefaultNamespace(namespace);
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#writeValue(java.lang.Object)
     */
    public void writeValue(Object value)
    {
        try
        {
            if ( value != null )
                writer.writeCharacters( value.toString() );
        }
        catch ( XMLStreamException e )
        {
            // TODO: I'm still not sure what should happen here.
            throw new XFireRuntimeException("Error writing document.", e);
        }
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#getWriter(java.lang.String)
     */
    public MessageWriter getChildWriter(String name)
    {
        return new LiteralWriter(writer, name, namespace);
    }

    public MessageWriter getChildWriter(String name, String ns)
    {
        return new LiteralWriter(writer, name, ns);
    }
    
    public String getNamespace()
    {
        return namespace;
    }

    public void setNamespace(String namespace)
    {
        this.namespace = namespace;
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsCalendar(java.util.Calendar)
     */
    public void writeValueAsCalendar(Calendar calendar)
    {
         DateUtils.formatDateTime( calendar.getTime() );
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsInt(java.lang.Integer)
     */
    public void writeValueAsInt(Integer i)
    {
        writeValue( i.toString() );
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsDate(java.util.Date)
     */
    public void writeValueAsDateTime(Date date)
    {
         writeValue( DateUtils.formatDateTime(date) );
    }

    /**
     * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsDate(java.util.Date)
     */
    public void writeValueAsDate(Date date)
    {
        DateUtils.formatDate( calendar.getTime() );
    }

	/**
	 * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsDouble(java.lang.Double)
	 */
	public void writeValueAsDouble(Double d)
	{
		writeValue( d.toString() );
	}

	/**
	 * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsLong(java.lang.Long)
	 */
	public void writeValueAsLong(Long l)
	{
		writeValue( l.toString() );
	}

	/**
	 * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsFloat(java.lang.Float)
	 */
	public void writeValueAsFloat(Float f)
	{
		writeValue( f.toString() );
	}

	/**
	 * @see org.codehaus.xfire.java.message.MessageWriter#writeValueAsBoolean(boolean)
	 */
	public void writeValueAsBoolean(boolean b)
	{
		writeValue( Boolean.toString(b) );
	}

    public void close()
    {
        try
        {
            writer.writeEndElement();
        }
        catch ( XMLStreamException e )
        {
            // TODO: I'm still not sure what should happen here.
            throw new XFireRuntimeException("Error writing document.", e);
        }
    }


}
