package org.codehaus.xfire.java.types;

import java.io.ByteArrayOutputStream;
import java.io.StringReader;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import org.codehaus.xfire.java.AbstractJavaTest;
import org.codehaus.xfire.java.mapping.TypeMappingRegistry;
import org.codehaus.xfire.java.message.LiteralReader;
import org.codehaus.xfire.java.message.LiteralWriter;
import org.codehaus.xfire.java.test.SimpleBean;
import org.codehaus.xfire.java.type.ArrayType;
import org.codehaus.xfire.java.type.BeanType;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * TypeTest
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class TypeTest
    extends AbstractJavaTest
{
    public void testBeanType() throws Exception
    {
        XMLOutputFactory ofactory = XMLOutputFactory.newInstance();
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        XMLStreamWriter writer = ofactory.createXMLStreamWriter(bos);

        TypeMappingRegistry tmr = getRegistry();
        
        SimpleBean bean = new SimpleBean();
        bean.setBleh("bleh");
        bean.setHowdy("howdy");
        
        registerSimpleBeanType(tmr);
        
        BeanType bt = (BeanType) tmr.getDefaultTypeMapping().getType( SimpleBean.class );

        LiteralWriter lwriter = new LiteralWriter(writer, "SimpleBean", "urn:Bean");
        bt.writeObject( bean, lwriter );
        lwriter.close();
        
        writer.close();
        
        System.out.println(bos.toString());
        XMLInputFactory factory = XMLInputFactory.newInstance();
        XMLStreamReader reader = factory.createXMLStreamReader( new StringReader(bos.toString()) );
        while ( reader.getEventType() != XMLStreamReader.START_ELEMENT )
            reader.next();
                
        SimpleBean readBean = (SimpleBean) bt.readObject( new LiteralReader(reader) );
        assertNotNull( readBean );
        assertEquals( "bleh", readBean.getBleh() );
        assertEquals( "howdy", readBean.getHowdy() );

        Document doc = DocumentHelper.createDocument();
        Element root = doc.addElement("root");
        
        bt.writeSchema( root );
 
        // TODO: run XPath tests on Schema
    }
    
    /**
     * @param tmr
     * @return
     */
    private void registerSimpleBeanType(TypeMappingRegistry tmr)
    {
        tmr.getDefaultTypeMapping().register( SimpleBean.class, 
                                                        new QName("urn:SimpleBean","SimpleBean"),
                                                        BeanType.class );
    }

    private void registerArrayType(TypeMappingRegistry tmr)
    {
        tmr.getDefaultTypeMapping().register( SimpleBean[].class, 
                                              new QName("urn:SomeBean" , "ArrayOfSimpleBean"),
                                              ArrayType.class );
    }
    
    public void testArrayType() throws Exception
    {
        XMLOutputFactory ofactory = XMLOutputFactory.newInstance();
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        XMLStreamWriter writer = ofactory.createXMLStreamWriter(bos);

        TypeMappingRegistry tmr = getRegistry();
        
        registerSimpleBeanType(tmr);
        registerArrayType( tmr );
        
        SimpleBean bean = new SimpleBean();
        bean.setBleh("bleh");
        bean.setHowdy("howdy");
       
        SimpleBean[] beanArray = new SimpleBean[] { bean, bean };

        ArrayType at = (ArrayType) tmr.getDefaultTypeMapping().getType( SimpleBean[].class );

        at.writeObject( beanArray, new LiteralWriter( writer, "SimpleBean", "urn:Bean" ) );
        writer.close();
        
        /* TODO: figure out why this doesn't work. It works when you're
         * actually reading/writing documents. I think it has something
         * to do with the reader.next() in the END_ELEMENT case in LiteralReader.

        XMLInputFactory factory = XMLInputFactory.newInstance();
        XMLStreamReader reader = factory.createXMLStreamReader( new StringReader(bos.toString()) );
        while ( reader.getEventType() != XMLStreamReader.START_ELEMENT )
            reader.next();

        Object out1 = at.readObject( new LiteralReader( reader ) );
        
        SimpleBean[] beans = (SimpleBean[]) out1;
        assertNotNull( beans );
        assertEquals( "bleh", beans[0].getBleh() );
        assertEquals( "howdy", beans[0].getHowdy() );

        Document doc = DocumentHelper.createDocument();
        Element root = doc.addElement("root");
        
        at.writeSchema( root );
        */
        // TODO: run XPath tests on Schema
    }
}
