package org.codehaus.xfire.java.wsdl;

import javax.wsdl.Definition;
import javax.wsdl.Message;
import javax.xml.namespace.QName;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.fault.SOAP11FaultHandler;
import org.codehaus.xfire.handler.SoapHandler;
import org.codehaus.xfire.java.AbstractJavaTest;
import org.codehaus.xfire.java.DefaultJavaService;
import org.codehaus.xfire.java.JavaServiceHandler;
import org.codehaus.xfire.java.test.Echo;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;


/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class DocumentWSDLTest
    extends AbstractJavaTest
{
    public void setUp() throws Exception
    {
        super.setUp();
        
        DefaultJavaService service = new DefaultJavaService();
        service.setName("Echo");
        service.setDefaultNamespace("urn:Echo");
        service.setServiceClass(Echo.class.getName());
        service.setSoapVersion(SOAPConstants.SOAP11_ENVELOPE_NS);
        service.setStyle(SOAPConstants.STYLE_DOCUMENT);
        service.setUse(SOAPConstants.USE_LITERAL);
        service.setFaultHandler(new SOAP11FaultHandler());
        service.setWSDLBuilder(new JavaWSDLBuilder( getXFire().getTransportManager() ));
        
        SoapHandler handler = new SoapHandler(new JavaServiceHandler());
        
        service.setServiceHandler( handler );
        
        service.setTypeMappingRegistry(getRegistry());
        
        service.initialize();
        
        getServiceRegistry().register(service);
    }
    
    public void testWSDL() throws Exception
    {
        DefaultJavaService echo = (DefaultJavaService) getServiceRegistry().getService("Echo");
        
        DocumentWSDL wsdl = (DocumentWSDL) echo.getWSDL();
        
        Definition def = wsdl.getDefinition();
        
        wsdl.write( System.out );
        
        Message request = def.getMessage( new QName("urn:Echo", "echoRequest") );
        
        assertNotNull( request );
        assertNotNull( request.getPart( "in0" ) );
        assertEquals( "in0", request.getPart( "in0" ).getElementName().getLocalPart() );
        
        Message response = def.getMessage( new QName("urn:Echo", "echoResponse") );
        
        assertNotNull( response );
        assertNotNull( response.getPart( "out" ) );
        assertEquals( "out", response.getPart( "out" ).getElementName().getLocalPart() );
        
        Document doc = wsdl.getDocument();

        addNamespace( "wsdl", WSDL.WSDL11_NS );
        addNamespace( "xsd", SOAPConstants.XSD );

        assertValid( "/wsdl:definitions/wsdl:types", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:element[@name=\"in0\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:element[@name=\"out\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:element[@name=\"in0\"][@type=\"xsd:string\"]", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema/xsd:element[@name=\"out\"][@type=\"xsd:string\"]", doc );
    }
}
